/***************************************************************************
                          hlist.h  -  description
                             -------------------
    begin                : jue jul 10 2003
    copyright            : (C) 2003 by Napsis S.L.
    email                : support@napsis.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HLIST_H
#define HLIST_H

#include <qptrlist.h>
#include <qdict.h>

/**
  *@author Carlos Manzanedo
  */

template  <class type>
class HList{
public: 
     HList(unsigned int _size = 17, bool case_sensitive = true){
          list = new QPtrList<type>;
          dict = new QDict<type>(_size,case_sensitive);

          list->setAutoDelete(false);
          dict->setAutoDelete(false);
     }
     
     ~HList(){
          this->clear();
          delete list;
          delete dict;          
	}
     
     int count(){ return list->count();}
     bool isEmpty(){ return list->count();}
     unsigned int size(){return list->count();}
     void resize(unsigned int new_size){ dict->resize(new_size);}

     int posOf(const type *item){
	     int count,i;
	     type *aux_item = 0;
	     
	     if (!item)
		     return (-1);
	     
	     count = list->count();
	     for(i=0;i<count;i++){
                     aux_item = list->at(i);
		     if (item == aux_item)
                              return i;
	     }
	     return (-1);
     }

     int posOf(const QString& key){
	     type *aux_item=0;

	     aux_item = this->find(key);
	     return posOf(aux_item);
	     
     }

     void clear(){
               dict->clear();
               list->clear();

     }

     void insert(unsigned int pos, const type *item,const QString& key){
          list->insert(pos,item);
          dict->insert(key,item);
     }

     void append(const type *item,const QString& key){
          list->append(item);
          dict->insert(key,item);
     }

     void prepend(const type *item, const QString& key){
          list->preppend(item);
          dict->insert(key,item);
     }

     bool remove(unsigned int pos){
          type *ob = 0;
          type *aux= 0;
          ob = list->at(pos);
          QDictIterator<type> it(*dict);
          for(; aux = it.current(); ++it ){
               if (ob == aux){
                    dict->remove(it.currentKey());
                    break;
               }
          }
          return list->remove(pos);
     }

     bool remove(const QString& key){
             bool ret;
             int count,i;
             type *ob = 0;
             type *aux = 0;

             ob = dict->find(key);
             if (!ob)
                     return false;
             dict->remove(key);
             if (moreThanOnce(ob))
                     return true;
  
             count = list->count();
             for(i=0;i<count;i++){
                     aux = list->at(i);
                     if (ob == aux){
                             ret = list->remove(list->at(i));
                             return true;
                     }
             }
             return false;
     }

     
     bool removeFirst(){return this->remove(0);}

     bool removeLast(){return this->remove(list->count()-1);}

     bool remove(){
          int pos;
          pos = list->at();
          return  this->remove(pos);
     }

     type* getFirst(){
          return list->first();
     }

     type* getLast(){
          return list->last();
     }

     type* at(unsigned int pos){
          return list->at(pos);
     }
     int at(){
          return list->at();
     }
     
     type* at(const QString& key ){
          return this->find(key);
     }

     type* next(){return list->next();}

     type* previous(){return list->previous();}

     type* current(){return list->current();}

     type* find(const QString& key){return dict->find(key);}
     type* take(const QString &key){
          type * ob=0;
          bool aux_autodelete;
          ob = dict->find(key);

          if (!ob)
               return NULL;
          aux_autodelete = list->autoDelete();

          list->setAutoDelete(false);
          this->remove(key);
          list->setAutoDelete(aux_autodelete);
          return ob;
     }

     bool autoDelete(){return list->autoDelete();}
     void setAutoDelete(bool enable){list->setAutoDelete(enable);}

     type* operator[] (const QString& key){return find(key);}
     type* operator[] (unsigned int pos){return list->at(pos);}


     void statistics(){dict->statistics();}

protected:
        bool moreThanOnce(const type *item){
                type *aux_item;
                int c=0;
                int count,i;
                count = list->count();
                for(i=0;i<count;i++){
                        aux_item = list->at(i);
                        if (aux_item == item)
                                c++;
                        if (c>=2)
                                return true;
                }
                return false;
        }


     
     QPtrList <type>  *list;
     QDict <type> *dict;
};

#endif
